
#ifndef MAIN677_H
#define MAIN677_H



/** Defines **/

// I2C Comms.
#define DEVICE_ADDR		0x72					// Device's I2C Address

// LED/Basic
#define	ON				0						// LED ON Definition  (Active Low)
#define OFF				1						// LED OFF Definition (Active Low) 
#define	CLEAR			0						// Bit Clear Definition
#define SET				1						// Bit Set Definition

// Capacitive
#define NUM_BTTNS		4						// Number of capacitive buttons
#define AVG_DELAY		8
// Percentages as: 50 = 5.0%, 273 = 27.3%
#define PCT_FULLON		50						// Percentage on a button must be pressed (or greater)
#define PCT_FULLOFF		33						// Percentage off for considered a 'off'  (or less)
#define PCT_HALFON		37						// Percentage on  for half press
#define PCT_HALFOFF		31						// Percentage off for half press
#define PCT_AVG			(PCT_HALFOFF-2)			// Percent range with which to avg (adjust to environmental factors, etc.)


// Debounce Defines
#define DBMAX		6													// See debounce code use.. quick expl. =x2 number of repetitive hits of sensor
#define NVAL		0
#define NEVAL		1
#define EVAL		2
#define SEVAL		3
#define SVAL		4
#define SWVAL		5
#define WVAL		6
#define NWVAL		7
#define ALLOFF		8



/** Declare Pinouts **/

// I2C Comms.
#define TRISSDA			TRISB4					// TRIS pins for I2C lines.
#define TRISSCL			TRISB6

// LED Outputs
#define LED_N			RB5						// LEDs to provide feedback to user
#define LED_NE			RA2						// when a key is pressed.
#define LED_E			RA0
#define LED_SE			RA4
#define LED_S			RC5
#define LED_SW			RC6
#define LED_W			RC7
#define LED_NW			RB7


//////////////////////////////////////////////////////////////////////


#ifndef TYPEDEFS_MAIN_H
#define TYPEDEFS_MAIN_H
// To make global must make typedefs... slightly awkward.

// Flag variable with SSP Busy Flag bit.
typedef struct {
						char	SSPBF:1;		// Flag if SSP Module is busy (Comm in progress)
						char	MATCHEDPRESS:1;	// Current button flag matches the actively pressed button
						char	BTNVALID:1;
						char	AVERAGESTOP:1;
						char	SLEEP:1;
					} FFlags;


// Flags for Capacitive Sensing, BTNx indicates a button down. 
// Cardinal directions will be set as combinations of BTN0, BTN1, BTN2, BTN3
typedef	struct {
						char	BTN0:1;			// { byte 0 }
						char	BTN1:1;
						char	BTN2:1;
						char	BTN3:1;
						char	BTN0H:1;
						char	BTN1H:1;
						char	BTN2H:1;
						char	BTN3H:1;
						char	N:1;
						char	S:1;
						char	E:1;
						char	W:1;
						char	NE:1;			// { byte 1 }
						char	NW:1;
						char	SE:1;
						char	SW:1;
					} BButtons;

#endif /* TYPEDEFS_MAIN_H */


//////////////////////////////////////////////////////////////////////


/** Variables **/

extern FFlags Flags;							// Flag variable with SSP busy flag bit.
extern BButtons Buttons;						// Flags for capacitive sensing, indicates a button down

// Debounce Variables
extern unsigned char	DebounceCount;
extern unsigned char	Last;

// I2C Communications
extern unsigned char	ByteCount;	
extern unsigned int		TempInt;
extern unsigned char	SSPCommand;
extern unsigned char	SSPIndex;
extern unsigned char	SSPSize;
extern unsigned char	IndexCount;


// Capacitive Variables
extern long				percent;
extern unsigned char	AvgIndex;

extern unsigned int		RAW	   [NUM_BTTNS];
extern unsigned int		AVERAGE[NUM_BTTNS];
extern unsigned int		TRIP   [NUM_BTTNS];
extern unsigned int		GUARD  [NUM_BTTNS];

extern unsigned char	FIRST;													// first variable to 'discard' first N samples
extern unsigned char	INDEX;													// Index of what button is being checked.

extern unsigned int		VALUE;													// current button value
extern unsigned int		BIGVAL;
extern unsigned int		SMALLAVG;







/* Prototypes */

void Init(void);
void CapInit(void);
void CapISR(void);
void SSPInit(void);
void SSPISR(void);				
void SetNextChannel(void);			
void RestartTimers(void);
static void interrupt isr(void);				// Declare functions called from isr above for best context saving.



void LoadSSPBUF(void);
void UnloadSSPBUF(void);
















#endif /* MAIN677_H */
