//******************************************************************************
// Software License Agreement
//
// The software supplied herewith by Microchip Technology
// Incorporated (the "Company") is intended and supplied to you, the
// Companys customer, for use solely and exclusively on Microchip
// products. The software is owned by the Company and/or its supplier,
// and is protected under applicable copyright laws. All rights are
// reserved. Any use in violation of the foregoing restrictions may
// subject the user to criminal sanctions under applicable laws, as
// well as to civil liability for the breach of the terms and
// conditions of this license.
//
// THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
// TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
// IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
//******************************************************************************


#include	"main887.h"
#include	"ssp.h"
#include	"display.h"
#include	"audio.h"

__CONFIG(INTIO & WDTDIS & PWRTEN & MCLRDIS & UNPROTECT & BORDIS & LVPDIS & IESODIS & FCMDIS);


// Defines
// Pinouts
// .. See header file


/** Variables **/

// Debug
unsigned long 	tep;

// I2C Communications
unsigned char	ByteCount;	
unsigned int	TempInt;
unsigned char	SSPCommand;
unsigned char	SSPIndex;
unsigned char	SSPSize;
unsigned char	IndexCount;

unsigned char	AvgIndex;
unsigned char	SliderIndex;

FFlags Flags;
BButtons Buttons;

bank2 unsigned int	RAW	   [NUM_BTTNS];
bank2 unsigned int	AVERAGE[NUM_BTTNS];		// Can't fit all in one bank. Only 96 bytes per Banks 2|3, (Max 24 bttns)
bank3 unsigned int	GUARD  [NUM_BTTNS];
bank3 unsigned int	TRIP   [NUM_BTTNS];		//  "


unsigned char	FIRST;						// first variable to 'discard' first N samples
unsigned char	INDEX;						// Index of what button is being checked.

unsigned int	VALUE;						// current button value
unsigned int	BIGVAL;						// current button bigval
unsigned int	SMALLAVG;					// current button smallavg

unsigned char	option_save;				// save/restore variable for option register in sleep mode.

// Helper variables
unsigned int		j;						// Loop variable, not for use inside ISR.



/** Constants **/

// 0x94/0xA0 = Cin0-    ,   0x95/0xA1 = Cin1-   (Sets comparator operation and proper channel.)
const unsigned char	COMP1[2]			= {0x94, 0x95};				// comparator preset values for each button
const unsigned char	COMP2[2]			= {0xA0, 0xA1};




/*====================================================================
========================  PROGRAM CODE  ==============================
====================================================================*/

/*....................................................................
. 	main()
.
.		Entry point of program.
....................................................................*/
void main(void) {



	INIT();											// Launch Device Initializations
	ClearK_LEDs();									// Init all Keypad LEDs to OFF state
	SetSliderLEDs(1);								// Init Slider Minimum LED to ON state
	Flags.SLEEP = 0;								// Initial Sleeping State (0=off)





	while (1) {										// Loop forever, illum LED if button is down.


		// Normal Awake Operation

		// Illuminate Keypad LEDs
		K_LED1 = (Buttons.BTN1 == 1) ? ON : OFF;	// (if : else)
		K_LED2 = (Buttons.BTN2 == 1) ? ON : OFF;
		K_LED3 = (Buttons.BTN3 == 1) ? ON : OFF;
		K_LED4 = (Buttons.BTN4 == 1) ? ON : OFF;
		K_LED5 = (Buttons.BTN5 == 1) ? ON : OFF;
		K_LED6 = (Buttons.BTN6 == 1) ? ON : OFF;
		K_LED7 = (Buttons.BTN7 == 1) ? ON : OFF;
		K_LED8 = (Buttons.BTN8 == 1) ? ON : OFF;
		K_LED9 = (Buttons.BTN9 == 1) ? ON : OFF;
		K_LED0 = (Buttons.BTN0 == 1) ? ON : OFF;

		// Beep if needed
		if ( !Flags.BUZZDONE && (Buttons.BTN0 || Buttons.BTN1 || Buttons.BTN2 || Buttons.BTN3 || Buttons.BTN4 || Buttons.BTN5 || Buttons.BTN6 || Buttons.BTN7 || Buttons.BTN8 || Buttons.BTN9)) {
			// If a buzz is not complete, and any button was pressed fire off a beep.
			Buzz();
		}

	
		
		// Illuminate Slider Position LEDs
		if (INDEX == iSL1) {
			// To prevent flickering as code is set up, must display indicator for button
			// only after a scan of all slider buttons.  Otherwise displaying mid scan
			// may cause flickering as only partial data is present of a complete scan.

			// This implementation defualts to the higher value if a press is
			// detected on multiple positions.

			if (Buttons.SLIDER6 == 1)
				SetSliderLEDs(6);
			else if (Buttons.SLIDER5 == 1)
				SetSliderLEDs(5);
			else if (Buttons.SLIDER4 == 1)
				SetSliderLEDs(4);
			else if (Buttons.SLIDER3 == 1)
				SetSliderLEDs(3);
			else if (Buttons.SLIDER2 == 1)
				SetSliderLEDs(2);
			else if (Buttons.SLIDER1 == 1)
				SetSliderLEDs(1);						// Lowest position is default 
		}

	
		// Standby Operation (sleep)

		// ... 
		// Issue Sleep Command
		if (Buttons.PWR) {
			Flags.SLEEP = 1;
			Buttons.PWR = 0;

			if (START == 1 || BF == 1)
				while (STOP == 0);
			SetMasterMode();							// Set I2C for master mode to issue commands	
			ShutdownWheel();							// Issue wheel shutdown command
			SSPIE = 0;									// Turn off I2C comms


			// Showy stuff for going to sleep mode
			// ...
			SetK_LEDs();								// Set all lights on, then turn them off
			SetSliderLEDs(6);							// Set moving down for show
			for (j=0; j<0x3000; j++);					// momentary delay
			SetSliderLEDs(5);
			K_LED1 = K_LED3 = OFF;						// turn keypad led's off in a distinct pattern
			for (j=0; j<0x3000; j++);
			SetSliderLEDs(4);
			K_LED2 = OFF;				
			for (j=0; j<0x3000; j++);
			SetSliderLEDs(3);
			K_LED4 = K_LED6 = OFF;					
			for (j=0; j<0x3000; j++);
			SetSliderLEDs(2);
			K_LED5 = K_LED7 = K_LED9 = OFF;					
			for (j=0; j<0x3000; j++);
			SetSliderLEDs(1);
			K_LED8 = OFF;			
			for (j=0; j<0x3000; j++);
			SetSliderLEDs(99);							// Set all off for sleep
			K_LED0 = 0;									// Set last keypad LED off.
		}

		// Begin Sleep operation
		if (Flags.SLEEP) {
			// Set Channel for wake-up button
			SetChannel(iPWR);							// Set to wake-up based off power button


			WDTCON = 0b00001110;						// WDT Off, WDT Prescalar 1:4096  31khz/4096 --> T~=0.12sec
			WDTCON = 0b00000110;						// 1:256 wdt ps  256/31kHz = 8.25ms asleep
			for (j=0; j<NUM_BTTNS; j++)
				AVERAGE[j] = 0;							// Clear all averages (so on wake does not cause issue).
		}
		// Continued sleep operation
		while (Flags.SLEEP) {

			CLRWDT();						// Clear WDT
			WDTCON |= 0x01;					// Enable WDT
			SLEEP();						// Put part to sleep for ~ 0.25 sec
			NOP();
			NOP();
			NOP();							// Nops to ensure no extra code is run.
			NOP();
			// On wake, Disable WDT for normal operation
			WDTCON &= 0xFE;					// Disable WDT
					
					
		
			// Provide enough time to scan for a button.
			FIRST = 3;							// Discard first N readings to establish nominal
			for (j=0; j<100; j++)				// Allow to stabilize and ensure
				Buttons.PWR = 0;				// Ensure cleared on waking

			ClearK_LEDs();						// Init all Keypad LEDs to OFF state
			for (j=0; j<0xFFFF; j++) {    		// Counts 0x1000, estimated to be plenty of on time.
				if (Buttons.PWR == 1) {
					GIE = 0;					// temporarily disable interrupts for the show portion.
					Buttons.BTN0  = 0;
					Buttons.BTN1  = 0;
			 		Buttons.BTN2  = 0;
			 		Buttons.BTN3  = 0;
			 		Buttons.BTN4  = 0;
			 		Buttons.BTN5  = 0;
			 		Buttons.BTN6  = 0;			// Clear all button flags.
			 		Buttons.BTN7  = 0;
					Buttons.BTN8  = 0;
					Buttons.BTN9  = 0;
					Buttons.SLIDER1 = 0;
					Buttons.SLIDER2 = 0;
					Buttons.SLIDER3 = 0;
					Buttons.SLIDER4 = 0;
					Buttons.SLIDER5 = 0;
					Buttons.SLIDER6 = 0; 
					Buttons.PWR = 0;

					// Awake Wheel
					T0IE = 0;						// Wait till finished waking up wheel to allow cap interrupts
					if (START == 1 || BF == 1)
						while (STOP == 0);
					SetMasterMode();				// Set master mode to issue cmds
					WakeupWheel();					// Issue wakeup to wheel

					// LED Turn on sequence (for show)
					SetSliderLEDs(1);			// make lights go up on slider bar led's
					K_LED0 = ON;				// sequence keypad led's too.
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(2);			// Set moving up for show
					K_LED7 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(3);
					K_LED4 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(4);
					K_LED1 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(5);
					K_LED2 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(6);
					K_LED3 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(7);
					K_LED6 = ON;
					for (j=0; j<0x3000; j++);
					SetSliderLEDs(99);
					K_LED9 = ON;
					for (j=0; j<0x3000; j++);
					K_LED8 = ON;				// there are more keypad led's than slider
					for (j=0; j<0x3000; j++);
					K_LED5 = ON;				// so they get two of their own.
					for (j=0; j<0x3000; j++);


					// Begin resuming normal operation
					Flags.SLEEP = 0;			// Disable SLEEP flag
					FIRST = 34;				// Need to reinit all buttons cleared averages. 17butt*2ea.
					break;
				}
			}

			// If exiting sleep..
			if (Flags.SLEEP == 0) {
				// LEDs
				ClearK_LEDs();					// Init all Keypad LEDs to OFF state
				SetSliderLEDs(1);				// Init Slider Minimum LED to ON state				
				// Averages
				for (j=0; j<NUM_BTTNS; j++)
					AVERAGE[j] = 0;				// Clear all averages (so on wake does not cause issue).
				// Renable intpts
				SSPIE = 1;						// Re-enable i2c interrupt
				T0IE = 1;						// Allow cap interrupts, fully normal operation resumes
				GIE = 1;						// Re-enable interrupts on awake after the showy stuff.
				SetSlaveMode();					// Setup slave mode to talk to PC GUI (must be after wakeupwheel transmission complete)


			}

		}


	}
}







/*....................................................................
. INIT()
.
.		Initialization Routine to set up part's variables and
.	peripherals.
....................................................................*/
void INIT(void) {


	// Set Pin I/O Direction

	// Determining pin usage from schematic (help determine TRIS setting)
	//--------------------------------------------------------------------------
	//			7		6		5		4		3		2		1		0
	// PORTA	LED		LED		C2OUT	.		.		C2+		C12IN0-	C12IN1-
	// PORTB	.		.		.		.		MUXSEL	S2		S1		S0
	// PORTC	LED		LED		LED		SDA		SCL		BUZZ	LED		T1CKI
	// PORTD	.		LED		LED		LED		LED		LED		LED		LED
	// PORTE	X		X		X		X		.		LED		LED		LED
	//--------------------------------------------------------------------------

	// Set LEDs as outputs
	TRISA &= 0b00111111;
	TRISC &= 0b00011101;
	TRISD &= 0b10000000;
	TRISE &= 0b11111000;

	// Set Selector channels as outputs
	TRISB &= 0b11110000;

	// Set Buzzer as output
	TRISC &= 0b11111011;

	// Capacitive
	TRISA &= 0b11011111;		// C2OUT = output
	TRISA |= 0b00000011;		// Comparator inputs C12INx-, x=0,1  as inputs
	TRISC |= 0b00000001;		// T1CKI = input


	// SDA & SCL taken care of in ssp.h / ssp.c
	// ... 



	CapInit();					// Initialize Capacitive Sensing
	SSPInit();					// Initialize I2C Communications

		
	SliderIndex = 1;			// Index for sliders to start at 1

	GIE = 1;					// turn on the global interrupts once done w/inits
}






/*....................................................................
.	isr()
.
.		Interrupt Service Routine - is the function called when any
.	interrupt is received on the part.  The device context saves, 
.	and then redirects flow here. Program flow will be returned at
.	the end of this function.  The flagging interrupt MUST be
.	cleared in software! (or you will be stuck for-ev-er).
....................................................................*/
void interrupt isr(void)
{																		
	
	// Timer 0 Interrupt
	if (T0IF == 1 && T0IE == 1) {													// Timer 0 Overflow Occurred, 
		// Service Capacitive Routine
		// .. pending not interfering with I2C
		TMR1ON	= 0;																// stop Timer1
	
		if (Flags.SSPBF == 1)														// SSP is busy, and can not interrupt it.
			RestartTimers();														// Clear timers
		else 																		// SSP is not busy, do cap. stuff
			CapISR();																// Service Routine {resets/restarts tmrs}			
	}



	if (Flags.SLEEP && SSPIF == 1)
		SSPIF = 0;																	// In sleep mode clear all SSP comm intpts

	// SSP Interrupt
	if (SSPIF == 1 && SSPIE == 1)													// SSP Interrupt Detected
		SSPISR();


	// Timer Interrupt During Non-TMR0 Intpt?
	if (T0IF == 1)																	// Catch if timer interrupt during ssp?  Handled at end of ISR.
		RestartTimers();															// (Clears T0IF)

}





/*....................................................................
. RestartTimers()
.
.		Resets and restarts timers 0 & 1 used for capacitive sensing.
....................................................................*/
void RestartTimers(void) {

	TMR1L	= 0;								// reset Timer1
	TMR1H	= 0;								//   "     "
	TMR1ON	= 1;								// restart Timer1
	TMR0	= 0;								// reset Timer0
	T0IF	= 0;								// clear the interrupt
}





/*....................................................................
.	CapInit()
.
.		Performs Initializations for capacitive sensing.
....................................................................*/
void CapInit(void) {

	// Load defaults for buttons
	for (INDEX=0; INDEX < NUM_BTTNS; INDEX++) {			// Load Default GB/TRP

		if (INDEX != 0 && INDEX < 11) {
			// Normal Buttons
			RAW[INDEX]		= 0;
			AVERAGE[INDEX] 	= 0;
			GUARD[INDEX]	= 60;
			TRIP[INDEX]		= 145;

			// Weaker sensors (make them a little more sensitive)
			if (INDEX == i2)
				TRIP[INDEX] = 120;
			else if (INDEX == i3)
				TRIP[INDEX] = 135;
			else if (INDEX == i5)
				TRIP[INDEX] = 120;

			// Too strong sensors
			if (INDEX == i0)
				TRIP[INDEX] = 165;

		} else if (INDEX != 0) {
			// Specifically Tailored Sliders
			if (INDEX == 11)
				TRIP[INDEX] = 110;		// Slider 1 is too insensitive, adjust to be more
			if (INDEX == 12)
				TRIP[INDEX] = 100;		// Slider 2 is far too insensitive,
			if (INDEX == 13)
				TRIP[INDEX] = 110;		// Slider 3 is too insensitive, adjust to be more
			if (INDEX == 14)
				TRIP[INDEX] = 115;
			if (INDEX == 15)
				TRIP[INDEX] = 130;
			if (INDEX == 16)
				TRIP[INDEX] = 130;		// Slider 6 is a little insensitive, and should be semi-dominant on 'down swipes'

		} else {
			// Power button index
			TRIP[INDEX] = 1500;
		}

	}



	FIRST	= 160;					// first pass flag to reach steady state average before enabling detection


	// All TRIS Registers set up in INIT() Routine
	// Capacitively this requires to set up C2OUT as o/p, and T1CKI=i/p, C12INx- pins used = analog i/p., C2+ pin = analog i/p
		
	OSCCON	= 0x7F;					// (Optional) Set for 8MHz internal clock.

	VRCON   = 0x87;					// setup the CVref for 2/3VDD
	CM1CON0 = COMP1[0];				// preset the comparators for first channel (and set appropriate enable bits)
	CM2CON0 = COMP2[0];
	INDEX	= 0;					// Init to start on button 0. { INDEX =button state machine state variable }
	CM2CON1 = 0x32;

	ANSEL   = 0x07;					// configure the button inputs for analog, along with the 1/4VDD reference input
	ANSELH  = 0x00;

	SRCON   = 0xF0;					// turn on the SR flip flop mode

	T1CON	= 0x06;					// enable timer1
	OPTION	= 0x84;					// setup the timer0 prescaler

	RestartTimers();				// Turn on and start timers

	INTCON	= 0;					// clear the interrupt register
	T0IE	= 1;					// turn on the timer0 interrupt
}





/*....................................................................
.	CapISR(void)
.
.		Capacitive service routine for ISR.  This function should be
.	called on each overflow of Timer0 which indicates a measurement
.	is ready to be taken.
.
.		It takes the measurement, determines if the button under test
.	is pressed or not (setting a flag accordingly), and it then will
.	average the new reading appropriately and then set the comparators
.	to scan the next button in sequence.
....................................................................*/
void CapISR(void) {


	// 1. Get the raw sensor reading.

	// ...
	VALUE = TMR1L + (unsigned int)(TMR1H << 8);								// Get the frequency count for the current sensor
	BIGVAL   = VALUE * 16;													// BIGVAL is current measurement left shifted 4 bits
	SMALLAVG = AVERAGE[INDEX] / 16;											// SMALLAVG is the current average right shifted 4 bits
	RAW[INDEX] = BIGVAL;													// RAW array holds the most recent BIGVAL values


	// 2. On power-up, reach steady-state readings

	// ...
	// During power-up, system must be initialized, set average ea. pass
	if (FIRST > 0) {
		FIRST--;															// Decr. N # times to establish startup
		AVERAGE[INDEX] = BIGVAL;											// During start up time, set Average each pass.
		if (!Flags.SLEEP)
			SetNextChannel();
		RestartTimers();													// Restart the timers
		return;																// .. and exit
	}


	// 3. Is a keypad button pressed?

	// ...
	if (BIGVAL < (AVERAGE[INDEX] - TRIP[INDEX])) {
		// Allow only 1 button pressed down at a time
		if (!(Buttons.BTN0 || Buttons.BTN1 || Buttons.BTN2 || 
			  Buttons.BTN3 || Buttons.BTN4 || Buttons.BTN5 || 
			  Buttons.BTN6 || Buttons.BTN7 || Buttons.BTN8 || Buttons.BTN9)) {

			switch(INDEX) {
					case i0:	Buttons.BTN0 = 1; Flags.BUZZDONE = 0;	break;	// Keypad 0(#10), 1-9 = Index 1-10
					case i1:	Buttons.BTN1 = 1; Flags.BUZZDONE = 0;	break;
					case i2:	Buttons.BTN2 = 1; Flags.BUZZDONE = 0;	break;	// Clear Buzzdone whenever a new button
					case i3:	Buttons.BTN3 = 1; Flags.BUZZDONE = 0;	break;	// press is detected to allow
					case i4:	Buttons.BTN4 = 1; Flags.BUZZDONE = 0;	break;	// buzzer logic to make a beep.
					case i5:	Buttons.BTN5 = 1; Flags.BUZZDONE = 0;	break;
					case i6:	Buttons.BTN6 = 1; Flags.BUZZDONE = 0;	break;
					case i7:	Buttons.BTN7 = 1; Flags.BUZZDONE = 0;	break;
					case i8:	Buttons.BTN8 = 1; Flags.BUZZDONE = 0;	break;
					case i9:	Buttons.BTN9 = 1; Flags.BUZZDONE = 0;	break;
			}
		}
	}

	// 3b. Power button pressed?

	// ...
	if (INDEX == iPWR && (BIGVAL < (AVERAGE[INDEX] - TRIP[INDEX])) ) {
		Buttons.PWR = 1;
	}






	// 4. Is a slider button pressed?

	// ...
	if ( (BIGVAL < (AVERAGE[INDEX] - TRIP[INDEX])) ) {
		// Is BIGVAL below { AVERAGE - TRIP } ? .. Yes
	
		// Button is down. 
			
		// Set flag for button that is pressed.
		switch(INDEX) {
			case iSL1:	Buttons.SLIDER1 = 1;							// Slider 1-6   = Index 11-16	
						Buttons.SLIDER2 = 0;
						Buttons.SLIDER3 = 0;
						Buttons.SLIDER4 = 0;
						Buttons.SLIDER5 = 0;
						Buttons.SLIDER6 = 0;
						SliderIndex   = 1;
						break;
			case iSL2:	Buttons.SLIDER1 = 0;	
						Buttons.SLIDER2 = 1;
						Buttons.SLIDER3 = 0;
						Buttons.SLIDER4 = 0;
						Buttons.SLIDER5 = 0;
						Buttons.SLIDER6 = 0;
						SliderIndex   = 2;								// Will set LEDs, pulling down oscillator freq,
						break;											// but will do so before averages of buttons below
			case iSL3:	Buttons.SLIDER1 = 0;							// can respond.  Setting the SliderIndex value
						Buttons.SLIDER2 = 0;
						Buttons.SLIDER3 = 1;
						Buttons.SLIDER4 = 0;
						Buttons.SLIDER5 = 0;
						Buttons.SLIDER6 = 0;
						SliderIndex   = 3;								// Shows the highest level of on value, and will
						break;											// allow averaging to be done on buttons beneath it
			case iSL4:	Buttons.SLIDER1 = 0;							// so that they do not become stuck.
						Buttons.SLIDER2 = 0;
						Buttons.SLIDER3 = 0;
						Buttons.SLIDER4 = 1;
						Buttons.SLIDER5 = 0;
						Buttons.SLIDER6 = 0;
						SliderIndex   = 4;
						break;
			case iSL5:	Buttons.SLIDER1 = 0;	
						Buttons.SLIDER2 = 0;
						Buttons.SLIDER3 = 0;
						Buttons.SLIDER4 = 0;
						Buttons.SLIDER5 = 1;
						Buttons.SLIDER6 = 0;
						SliderIndex   = 5;
						break;
			case iSL6:	Buttons.SLIDER1 = 0;	
						Buttons.SLIDER2 = 0;
						Buttons.SLIDER3 = 0;
						Buttons.SLIDER4 = 0;
						Buttons.SLIDER5 = 0;
						Buttons.SLIDER6 = 1;
						SliderIndex   = 6;
						break;
			default:	break;
	
		}
	}

	


	// 5. Is a button unpressed?

	// ...
	if (BIGVAL > (AVERAGE[INDEX] - TRIP[INDEX] + 32)) {
		// Is BIGVAL above { average - (trip + hysteresis) } ? .. Yes

		// Button is up.

		switch(INDEX) {
			case i0:	Buttons.BTN0 = 0;	break;
			case i1:	Buttons.BTN1 = 0;	break;
			case i2:	Buttons.BTN2 = 0;	break;
			case i3:	Buttons.BTN3 = 0;	break;
			case i4:	Buttons.BTN4 = 0;	break;
			case i5:	Buttons.BTN5 = 0;	break;
			case i6:	Buttons.BTN6 = 0;	break;
			case i7:	Buttons.BTN7 = 0;	break;
			case i8:	Buttons.BTN8 = 0;	break;
			case i9:	Buttons.BTN9 = 0;	break;
			case iSL1:	Buttons.SLIDER1 = 0;	break;
			case iSL2:	Buttons.SLIDER2 = 0;	break;
			case iSL3:	Buttons.SLIDER3 = 0;	break;
			case iSL4:	Buttons.SLIDER4 = 0;	break;
			case iSL5:	Buttons.SLIDER5 = 0;	break;
			case iSL6:	Buttons.SLIDER6 = 0;	break;
			case iPWR:	Buttons.PWR  = 0;	break;
			default:	break;
		}
	}


	// 6. Implement quick-release for a released button

	// ...
	if (BIGVAL  > AVERAGE[INDEX]) {
		AVERAGE[INDEX] = AVERAGE[INDEX] + (VALUE - SMALLAVG);		// Average in higher value every pass (not on timed delays), averaging protects against large spikes.
	}


	// 7. Average in the new value

	// ...
	// Always Average (all buttons)
	if (AvgIndex < AVG_DELAY)		AvgIndex++;						// Counting 0..8 has effect of every 9th count cycling the next button.
	else							AvgIndex = 0;					// Counting 0..4 will average faster and also can use 0..4*m, m=0,1,2,3..
	
	if (AvgIndex == AVG_DELAY) {
		// Average in raw value.
		AVERAGE[INDEX] = AVERAGE[INDEX] + (VALUE - SMALLAVG);		
	}

	 
	// Determine next sensor to test.
	if (!Flags.SLEEP)												// Don't change channels during sleep
		SetNextChannel();

	// Restart timers for next pass
	RestartTimers();			
}






/*....................................................................
.	SetNextChannel()
.
.		The SetNextChannel function will set the next channel in
.	sequence to be scanned based off of the current index value.
....................................................................*/
void SetNextChannel(void) {

	// 8. Complete code to change channels
	// ...

	// Determine next sensor to test.
	if ( INDEX <= (NUM_BTTNS - 1) ) {				// if (less than total index maximum) ..
		INDEX++;									// Incr. to test next button if not last index value
		MUXSEL = (INDEX <= 8) ? 1 : 0;				// (if:else)
		CTL_PORT &= 0xF8;							// Clear Low 3 bits
		CTL_PORT |= ( ( (INDEX-1) % 8) & 0x07 );	// Set 3 LSb's properly for MUXes.
	
	} else {
		INDEX  = 0;									/// Wrap around if just tested the last button
		MUXSEL = 1;									// Reset MUXSEL
		CTL_PORT = CTL_PORT & 0xF8;					// Reset S<2:0>
	} 

	// Configure comparators for the new sensor
	if (INDEX == 0) {
		CM1CON0 = COMP1[0];							// Power button, Index 0 is on CIN0-
		CM2CON0 = COMP2[0];
	} else {
		CM1CON0 = COMP1[1];							// All others (MUXes), are on CIN1-
		CM2CON0 = COMP2[1];
	}

}



/*....................................................................
.	SetChannel()
.
.		The SetChannel function will set the next channel in
.	sequence to be scanned based off of the supplied index value.
.	It assumes a valid index value.
....................................................................*/
void SetChannel(char idx) {

	INDEX = idx;								// Set state variable index.

	// Configure muxes as needed
	MUXSEL = (INDEX <= 8) ? 1 : 0;				// (if:else)
	CTL_PORT &= 0xF8;							// Clear Low 3 bits
	CTL_PORT |= ( ( (INDEX-1) % 8) & 0x07 );	// Set 3 LSb's properly for MUXes.
	
	// Configure comparators for the new sensor
	if (INDEX == 0) {
		CM1CON0 = COMP1[0];							// Power button, Index 0 is on CIN0-
		CM2CON0 = COMP2[0];
	} else {
		CM1CON0 = COMP1[1];							// All others (MUXes), are on CIN1-
		CM2CON0 = COMP2[1];
	}

}




/*....................................................................
.	ShutdownWheel()
.
.		Issues I2C command to 16F677 on SDA and SCL lines to put it
.	in sleep.  
....................................................................*/
void ShutdownWheel(void) {
	
	// Must send the following command to the 16F677 to shut it down
	// [S] DAw CommandByte DummyByte DummyByte [P]
	
	SSPIE = 0; 							// For simple cmd, avoid req'ng use of intpts
	SEN = 1;							// Issue start bit
	while (SEN);						// wait for start bit completion
	SSPBUF = _16F677_DEVICEADDR;		// Send 677 DA with write bit
	while (RW);							// Wait while transmission in progress
	SSPBUF = _16F677_SHUTDOWN;			// Issue shutdown cmd
	while (RW);							// Wait while transmission in progress
	SSPBUF = 0x00;						// Send 0x00 to comply with formatting
	while (RW);							// Wait while transmission in progress
	SSPBUF = 0X00;						// Send 0x00 to comply with formatting
	while (RW);							// Wait while transmission in progress
	PEN = 1;							// Issue stop bit
	while (PEN);						// Wait for completion, EOT
	SSPIE = 1;							// Re-enable intpts for GUI communication

}



/*....................................................................
.	WakeupWheel()
.
.		Issues I2C command to 16F677 on SDA and SCL lines to wake it
.	from sleep.  
....................................................................*/
void WakeupWheel(void) {

	// Must send the following command to the 16F677 to shut it down
	// [S] DAw CommandByte DummyByte DummyByte [P]

	SSPIE = 0; 							// For simple cmd, avoid req'ng use of intpts
	SEN = 1;							// Issue start bit
	while (SEN);						// wait for start bit completion
	SSPBUF = _16F677_DEVICEADDR;		// Send 677 DA with write bit
	while (RW);							// Wait while transmission in progress
	SSPBUF = _16F677_WAKEUP;			// Issue wakeup command
	while (RW);							// Wait while transmission in progress
	SSPBUF = 0x00;						// Send 0x00 to comply with formatting
	while (RW);							// Wait while transmission in progress
	SSPBUF = 0x00;						// Send 0x00 to comply with formatting
	while (RW);							// Wait while transmission in progress
	PEN = 1;							// Issue stop bit
	while (PEN);						// Wait for completion, EOT
	SSPIE = 1;							// Re-enable intpts for GUI communication

}


